'use strict';

var EventEmitter = require('events');
var util = require('util');

var Poll = require('poll');
var fs = require('fs');

var STDIN = 0;
var STDOUT = 1;

var CHUNK_SIZE = 1024;

function Serial() {
    EventEmitter.call(this);
    this._buffer = new Buffer(CHUNK_SIZE);
    this._readPoll = new Poll(STDIN);
}
util.inherits(Serial, EventEmitter);

Serial.prototype.startRead = function (callback) {
    var that = this;
    this._readPoll.start(Poll.READ_EVENT, function (error, event) {
        if (error) {
            callback && callback(error);
            return;
        }
        if (event === Poll.READ_EVENT) {
            var length = fs.readSync(STDIN, that._buffer, 0, CHUNK_SIZE, -1);
            // console.log('poll length is ', length);
            if (length > 0) {
                that.emit('data', that._buffer.slice(0, length));
            }
        }
    });
};

Serial.prototype.stopRead = function () {
    this._readPoll.stop();
};

Serial.prototype.write = function (data, callback) {
    fs.write(STDOUT, data, -1, callback);
};

module.exports = Serial;
