/* eslint-disable no-throw-literal, no-alert */

'use strict';

var $previousNavItem;
var previousFormSelector;

$('.nav').delegate('> li > a', 'click', function (event) {
    return activate($(event.target));
});

sendCommand('system.getNetStatus')
    .then(function (status) {
        var connected = renderStatusForm(status);
        var tabHash = location.hash || connected && '#status' || '#wifi';

        var firstActiveQuery = '.nav > li > a';
        var $firstActiveTarget;

        if (tabHash) {
            $firstActiveTarget = $(firstActiveQuery + '[href="' + tabHash + '"]');
        }

        if (!$firstActiveTarget || !$firstActiveTarget.length) {
            $firstActiveTarget = $(firstActiveQuery).first();
        }

        activate($firstActiveTarget);
    });

function activate($target) {
    var formSelector = $target.data('form');
    var $navItem = $target.parent();

    if (formSelector === previousFormSelector) {
        return;
    }

    if ($previousNavItem) {
        $previousNavItem.removeClass('active');
    }

    $navItem.addClass('active');

    $(previousFormSelector).hide();
    $(formSelector).show();

    $previousNavItem = $navItem;
    previousFormSelector = formSelector;
}

registerAction($('#settings-form-ap'), function () {
    var password = $('#ap-password').val();
    var confirmation = $('#ap-repassword').val();

    if (password.length < 8) {
        throw 'Password should NOT contain less than 8 characters.';
    }

    if (password !== confirmation) {
        throw 'The passwords you entered do not match.';
    }

    return sendCommand('system.setAP', { password: password });
});

registerAction($('#settings-form-wifi'), function () {
    var ssid = $('#wifi-ssid').val().trim();
    var password = $('#wifi-password').val();

    if (!ssid) {
        throw 'Please enter SSID.';
    }

    return sendCommand('system.setWifi', {
        ssid: ssid,
        password: password
    });
});

registerAction($('#settings-form-server'), function () {
    var httpPort = parseInt($('#server-http-port').val().trim(), 10);

    if (isNaN(httpPort) || httpPort < 1 || httpPort > 65535) {
        throw 'Please enter a valid port.';
    }

    return sendCommand('system.setServerSettings', {
        httpPort: httpPort
    });
});

function renderStatusForm(status) {
    var staStatus = status.sta;
    var ethStatus = status.eth;

    var items = [];

    if (staStatus.connected) {
        items.push({
            title: 'Wi-Fi Status',
            fields: [
                {
                    name: 'SSID',
                    value: staStatus.ssid
                },
                {
                    name: 'IP address',
                    value: staStatus.ip
                }
            ]
        });
    }

    if (ethStatus.connected) {
        items.push({
            title: 'Ethernet Status',
            fields: [
                {
                    name: 'IP address',
                    value: ethStatus.ip
                }
            ]
        });
    }

    var connected = !!items.length;

    if (!connected) {
        items.push({
            title: 'Status',
            fields: 'No network connections.'
        });
    }

    appendFormFields($('#settings-form-status'), items);

    return connected;
}

function appendFormFields($target, items) {
    var $fragment = $(document.createDocumentFragment());

    items.forEach(function (item) {
        var title = item.title;
        var fields = item.fields;

        $fragment.append($(document.createElement('h2')).text(title));

        if (typeof fields === 'string') {
            $fragment.append($(document.createElement('p')).text(fields));
            return;
        }

        fields.forEach(function (field) {
            var $value = $(document.createElement('input'))
                .prop('disabled', true)
                .val(field.value);

            var $label = $(document.createElement('label'))
                .text(field.name)
                .append($value);

            $fragment.append($label);
        });
    });

    $target.append($fragment);
}

function registerAction($form, action) {
    var $button = $form.find('button[type=submit]');

    var text = $button.text();
    var processingText = $button.data('processing-text');
    var doneText = $button.data('done-text');

    $button.click(function () {
        $button
            .prop('disabled', true)
            .text(processingText);

        Promise
            .resolve()
            .then(function () {
                return action();
            })
            .then(function () {
                $button.text(doneText);

                setTimeout(function () {
                    $button
                        .prop('disabled', false)
                        .text(text);
                }, 1000);
            }, function (reason) {
                if (typeof reason === 'string') {
                    alert(reason);
                } else {
                    alert('Error performing action: ' + (reason && reason.message));
                }

                $button
                    .prop('disabled', false)
                    .text(text);
            });
    });
}

function sendCommand(type, options) {
    options = options || {};

    options.type = type;

    return fetch('/api/call', {
        method: 'post',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(options),
        credentials: 'include'
    })
        .then(function (response) {
            return response.json();
        })
        .then(function (data) {
            if (data.error) {
                throw new Error(data.error.message);
            }

            return data.result;
        });
}
